@@ -25,12 +25,11 @@ setup_calendar.daily <- function(x, dir = "h", ...) {
     row_idx <- col_idx
     col_idx <- col_tmp
   }
-  cal_table <- data.frame(
+  dplyr::tibble(
     ROW = rlang::flatten_int(row_idx),
     COL = rlang::flatten_int(col_idx),
     PANEL = do.call("c", counter)
   )
-  return(cal_table)
 }
 
 setup_calendar.weekly <- function(x, dir = "h", ...) {
@@ -50,12 +49,7 @@ setup_calendar.weekly <- function(x, dir = "h", ...) {
     row_idx <- col_idx
     col_idx <- col_tmp
   }
-  cal_table <- data.frame(
-    ROW = row_idx,
-    COL = col_idx,
-    PANEL = counter
-  )
-  return(cal_table)
+  dplyr::tibble(ROW = row_idx, COL = col_idx, PANEL = counter)
 }
 
 setup_calendar.monthly <- function(x, dir = "h", sunday = FALSE, 
@@ -115,7 +109,7 @@ setup_calendar.monthly <- function(x, dir = "h", sunday = FALSE,
       .x = col_idx, .y = m_idx, function(.x, .y) .x + max_wks * (.y - 1)
     )
   }
-  cal_table <- data.frame(
+  dplyr::tibble(
     ROW = rlang::flatten_int(row_idx),
     COL = rlang::flatten_int(col_idx),
     MROW = rep.int(n_idx, days_x),
@@ -123,7 +117,6 @@ setup_calendar.monthly <- function(x, dir = "h", sunday = FALSE,
     PANEL = do.call("c", counter_date),
     MPANEL = rep.int(seq_len(nfacets), days_x)
   )
-  return(cal_table)
 }
 
 map2 <- function(.x, .y, .f, ...) {
